/************************************************************************
 * Spooky - a service framework for the Internet
 * DefaultInitializer.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.client;

import spooky.users.ejb.*;
import spooky.config.*;
import spooky.security.auth.*;

import javax.ejb.*;
import javax.naming.*;
import javax.rmi.*;
import org.w3c.dom.*;
import java.io.*;

/**
 * Class initializes the basic configurations to the database.
 *
 * @author  ekarlsso
 * @version 
 */
public class DefaultInitializer {

    /** Creates new DefaultInitializer */
    public DefaultInitializer() {
    }

    public static void main( String[] args )
    {
        try {
            String adminName = "administrator";
            String adminPass = "changeme";
            
            System.out.println("Spooky framework default settings.");
            
            InitialContext ctx = new InitialContext();
            Object boundObj = ctx.lookup("UserFacadeEJB");
            UserFacadeHome aHome = (UserFacadeHome)PortableRemoteObject.narrow( boundObj, UserFacadeHome.class );
            UserFacade userFace = aHome.create();
            
            //Now we will create the default admin user for the system
            System.out.println("Creating the admin user with the username \""+adminName+"\" and password \""+adminPass+"\"...");
            int userId = userFace.createUser(adminName, "Framework Administrator", adminPass );
            FrameworkUserPrincipal adminUserPrincipal = userFace.getUserPrincipal( adminName );
            
            //Next we must get the default admin groups name, so that we can create it and
            //add the default admin user to it. 
            System.out.println("Loading default information from config.xml...");
            FileInputStream configFileStream = new FileInputStream("config.xml");
            Document configDoc = ConfigFileDAO.loadFile( configFileStream );
            ConfigurationNode defaultConfigNode = ConfigFileDAO.loadDefaultNode( configDoc );
            String defaultAdminGroupName = defaultConfigNode.getAdminGroupName();
            
            System.out.println("Creating the default user group "+ defaultAdminGroupName);
            userFace.createUserGroup( defaultAdminGroupName, "/", "The default admin group in the system" );
            FrameworkGroupPrincipal groupPrincipal = userFace.getGroupPrincipal( defaultAdminGroupName );
            
            System.out.println("Adding the administrator to the default admin group");
            userFace.addUserToGroup( adminUserPrincipal, groupPrincipal  );
            System.out.println("Default user group and administrator added to the system");
            System.exit(0);
        }
        catch( Exception e )
        {
            System.err.println("***********************\nError occured:\n "+e.toString() );
            e.printStackTrace();
        }
    }
}
