/************************************************************************
 * Spooky - a service framework for the Internet
 * ConfigruationLoadException.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/

//Not added to the CVS

package spooky.config;

import java.util.*;


/**
 * Class represents a node of configuration information. The node groups the 
 * request mappings, view mappings and web model configrations under nodes, that
 * can be administrated by separate units.
 * 
 * @author  Erik Karlsson
 * @version 
 */
public class ConfigurationNode implements java.io.Serializable{

    public static String CONTEXT_NAME = "configration_node";
    private String nodeName;
    private String adminGroupName;
    private String currentPath;
    
    private HashMap eventMappings;
    private HashMap modelMappings;
    private HashMap viewMappings;
    
    /** Creates new Configurations */
    public ConfigurationNode( String name, String nameOfAdminGroup, HashMap events, HashMap models, HashMap views, String path ) throws ConfigurationLoadException {
    
        this.nodeName = name;
        this.adminGroupName = nameOfAdminGroup;
        this.eventMappings = events;
        this.viewMappings = views;
        this.modelMappings = models;
        this.currentPath = path;
    
    }

    public void setPath( String path ){
        this.currentPath = path;
    }
    
    public String getPath(){
        return currentPath;
    }
    
    /**
     * Returns the name of the configuration node
     *
     * @return name of the configuration node
     */
    public String getNodeName(){
        return this.nodeName;
    }
    
    /**
     * Returns the name of the adming group of this node.
     *
     * @return name of the adming group
     */
    public String getAdminGroupName(){
        return adminGroupName;
    }
    
    public HashMap getEventMappings() {
        return eventMappings;
    }
    
    public HashMap getModelMappings() {
        return modelMappings;
    }
    
    public HashMap getViewMappings() {
        return viewMappings;
    }    
    
    /**
     * Returns the name of the object
     */
    public String toString(){
        return this.nodeName;
    }
    
}
