/************************************************************************
 * Spooky - a service framework for the Internet
 * Configurations.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.config;
import java.util.*;
import org.w3c.dom.Document;

import javax.swing.*;
import javax.swing.tree.*;

/**
 * Class contains and manages the configurations of the Spooky 
 * framework. The configurations are stored to a tree alike 
 * container.
 *
 * @author  Erik Karlsson
 * @version 
 */
public class Configurations implements java.io.Serializable {

    public static String CONTEXT_NAME = "configurations";
    /*
    private HashMap eventMappings;
    private HashMap modelMappings;
    private HashMap viewMappings;
    */
    private JTree configurationTree;
    private ConfigurationNode defaultConfigNode;
    
    /** 
     * Creates new Configurations object. The configFileStream must 
     * point to valid configuration.xml file.
     *
     * @param configGileStream a stream to configuration.xml file 
     */
    public Configurations( java.io.InputStream configFileStream )throws ConfigurationLoadException 
    {
        Document configDoc = ConfigFileDAO.loadFile( configFileStream );
        this.defaultConfigNode = ConfigFileDAO.loadDefaultNode( configDoc );
        this.configurationTree = ConfigFileDAO.loadNodeMap( configDoc );   
    }

    public JTree getConfigurationTree(){
        return configurationTree;
    }
    
    /**
     * Checks if the node is the searched tree node. This is done
     * by comparing the name of the token and the name of the 
     * stored configuration node.
     *
     * @param treeNode the node to be check
     * @param name the token string
     * @return true if the token and node name match, else false
     */
    private boolean checkNode( DefaultMutableTreeNode treeNode, String name )
    {
        Object userObj = treeNode.getUserObject();
        ConfigurationNode confNode = (ConfigurationNode)userObj;
        
        if( confNode.getNodeName().equals( name ) )
            return true;
        else 
            return false;
    }
    
    /**
     * Method is used to search the configuration node for certain url 
     * request.s
     *
     * @return the configuration node if it was found, else <code>null</code>
     */
    public ConfigurationNode searchNode( String path )
    {
        StringTokenizer stringTok = new StringTokenizer( path, "/" );
        int amountOfTokens = stringTok.countTokens();
        amountOfTokens--;
        
        if( amountOfTokens <= 0 )
        {
            //Lets return the default nodes configuration
            //information!
            return defaultConfigNode;
        }
        
        TreeModel treeModel = configurationTree.getModel();
        DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode)treeModel.getRoot();
        String nodeValue2 = treeNode.getUserObject().toString();
        String nextToken = stringTok.nextToken();

        Enumeration children = treeNode.children();
            
        while( children.hasMoreElements() && amountOfTokens > 0)
        {
            DefaultMutableTreeNode tempNode = (DefaultMutableTreeNode)children.nextElement();
            String nodeValue = tempNode.getUserObject().toString();
            if( checkNode( tempNode, nextToken ) )
            {
                nextToken = stringTok.nextToken();
                treeNode = tempNode;
                amountOfTokens--;
                if( amountOfTokens == 0 ){
                    return (ConfigurationNode)treeNode.getUserObject();
                }
                children = treeNode.children();
            }
         }
         return null;
    }
    
}
