/************************************************************************
 * Spooky - a service framework for the Internet
 * EventProcessor.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.control.event;

import spooky.config.*;
import spooky.security.*;
import spooky.debug.*;
import spooky.exception.*;
import java.util.*;

/**
 *
 * @author  Erik Karlsson
 * @version 
 */
public class EventProcessor {

    private EventContext eventContext;
    private EventProcessorConfig eventConfig;
    private FrameworkContext securityContext;
    
    
    public EventProcessor()
    {
        SimpleTrace.printMessage("HttpEventProcessor::HttpEventProcessor()");
        this.eventConfig = null;
        this.eventContext = null;
    }
    
    /**
     * Inits the event processor with the event context and event processor
     * config objects. This method is called only once for the event processor
     * when it is initialized (and created) for the first time. The initialization
     * is made in an object that is instance of class that implements the
     * <code>EventHandler</code> interface.
     *
     * @param ec the <code>EventContext</code> of the enviroment where the
     *           processor is used. (Note. The HttpEventProcessor is used when
     *          the processor is in HTTP enviroment)
     * @param ecf The configurations for the current processor. 
     * @see spooky.config.EventProcessorConfig
     */
    public void init( EventContext ec, EventProcessorConfig ecf )
    {
        this.eventConfig = ecf;
        this.eventContext = ec;        
    }

    /**
     * Sets the security context of the current event processor. Note that
     * the security context is null if the current user is not authenticated.
     * So if the service requires that the security context is present it 
     * should be declared in the configuration file so that it requires
     * authentication. This ensures that the user is authenticated before
     * the even can be processed.
     *
     * @param securityContext the security context
     */
    public void setSecurityContext( FrameworkContext securityContext )
    {
        this.securityContext = securityContext;
    }
    
    /**
     * Returns the security context of the current authentication
     * session.
     *
     * @return FrameworkContext that represents the current security context
     */
    public FrameworkContext getSecurityContext()
    {
        return this.securityContext;
    }
    
    /**
     * Returns the event context. The event context contains the processors
     * context's object like http session, http request etc. objects. What the
     * context contains depends on the enviroment where the processor is
     * executed.
     *
     * @return the event context of the current enviroment
     */
    public EventContext getEventContext()
    {
        return this.eventContext;
    }
    
    
    public EventResponse createErrorResponse( String msg, String className, String methodName )
    {
        ErrorObject errorObject = new ErrorObject( msg, className, methodName);
        EventResponse eventResponse = new EventResponse();
        eventResponse.addErrorObject( errorObject );
        return eventResponse;          
    }
    
    /**
     *
     *
     */
    public EventProcessorConfig getEventProcessorConfig()
    {
        return this.eventConfig;
    }
    
    /**
     *  Method is called if the there isn't any action
     *  in the request.
     */
    public EventResponse pageLoadEvent(){
        EventResponse eventResponse = new EventResponse();
        return eventResponse;
    }
    
    public void cleanEvent()
    {
    }
    
    public void initEvent() throws EventProcessingException
    {
        int a = 0;
        if( a != 0 )
            throw new EventProcessingException();
    }
    
    public EventConfig getEventConfig( String eventName )
    {
        EventProcessorConfig eventConfig = getEventProcessorConfig();
        HashMap events = eventConfig.getEvents();
        EventConfig theEventConfig = (EventConfig)events.get( eventName ); 
        return theEventConfig;
    }    
}

