/************************************************************************
 * Spooky - a service framework for the Internet
 * EventResponse.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.control.event;

import java.util.*;
import spooky.exception.*;

/**
 *
 * @author  Erik Karlsson
 * @version 
 */
public class EventResponse implements java.io.Serializable {

    private Collection updatedModels;
    private HashMap attributesForModels;
    private String nextTemplate;
    private LinkedList errorList;
    private boolean empty;
    private boolean errors;
    
    /**
     *  Constructs a response object. This constructor should be used
     *  when an error is encountered. The constructor sets the object
     *  to be empty. 
     */
    public EventResponse() 
    {
        this.updatedModels = new ArrayList();
        this.attributesForModels = null;
        this.empty = true;
        this.nextTemplate = "";
        this.errors = false;
        errorList = new LinkedList();
    }

    public EventResponse( String nextTemplate ) 
    {
        this.updatedModels = new ArrayList();
        this.attributesForModels = null;
        this.empty = true;
        this.nextTemplate = nextTemplate;
        this.errors = false;
        errorList = new LinkedList();
    }    
    
    /**
     *  Constructs the response object. This constructor should be used
     *  by the service programmers.
     *
     *  @param updatedModels A collection of all updated models (their names)
     *  @param attributesForModels a has map that contains attributes for the
     *                              updated models. The name will indicate the
     *                              model and the object the attribute
     *  @param nextTemp next template to be shown.
     */
    public EventResponse( Collection updatedModels, HashMap attributesForModels, String nextTemp ) 
    {
        this.updatedModels =updatedModels;
        this.attributesForModels = attributesForModels;
        this.empty = false;
        this.nextTemplate = nextTemp;
        this.errors = false;
        this.errorList = new LinkedList();
    }

    /**
     *  Adds an error object to the response object.
     *  If the response contains any error objects
     *  the request will be interrupted. Note that
     *  the service programmer should also keep track
     *  on the transactions and uncommit them if 
     *  needed.
     *
     *  @param errorObject the object to be added
     */
    public void addErrorObject( ErrorObject errorObject ) 
    {
        errors = true;
        errorList.add( errorObject );
    }
    
    /**
     *  Returns error objects as a linked list.
     *
     *  @return error objects.
     */
    public LinkedList getErrorObjects()
    {
        return errorList;
    }
    /**
     *  Returns a collection of updated models. 
     *
     *  @return updated models
     */
    public Collection getUpdatedModels() {
        return this.updatedModels;
    }
    
    public HashMap getAttributesForModels() {
        return this.attributesForModels;
    }
    
    public Object getAttribute( String key ) {    
        return this.attributesForModels.get( key );
    }

    /**
     * Method returns true if no models was modified, 
     * else it will return false. 
     */
    public boolean empty(){
        
        if( updatedModels.size() < 1 )
            return true;
        else
            return false;
    }
    
    public String getNextTemplate(){
        return nextTemplate;
    }
    
    public void setErrors( boolean err ) {
        errors = err;
    }
    
    public boolean errors(){
        return errors;
    }
}
