/************************************************************************
 * Spooky - a service framework for the Internet
 * HttpEventProcessor.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.control.event;
import spooky.config.*;
import spooky.security.*;
import spooky.security.auth.*;
import spooky.debug.*;

import java.util.*;
import spooky.users.ejb.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletContext;

import javax.ejb.*;
import javax.naming.*;
import javax.rmi.*;
/**
 *
 * @author  ekarlsso
 * @version 
 */
public class HttpEventProcessor extends EventProcessor {

    private HttpSession httpSession;
    private HttpServletRequest httpServletRequest;
    private ServletContext servletContext;
    
    /** Creates new HttpEventProcessor */
    public HttpEventProcessor() 
    {
        super();
        SimpleTrace.printMessage("HttpEventProcessor::HttpEventProcessor()");
    }

    public void init( EventContext ec, EventProcessorConfig ecf )
    {
        super.init( ec, ecf );
        
        EventContext eventContext = super.getEventContext();

        httpSession = (HttpSession)eventContext.getObject( WebEnviroment.HTTP_SESSION_ID );
        httpServletRequest= (HttpServletRequest)eventContext.getObject( WebEnviroment.HTTP_REQUEST_ID );
        servletContext = (ServletContext)eventContext.getObject( WebEnviroment.SERVLET_CONTEXT_ID );      
    }
    
    public HttpSession getHttpSession()
    {
        return httpSession;
    }
    
    public HttpServletRequest getHttpRequest()
    {
        return httpServletRequest;
    }
    
    public ServletContext getServletContext()
    {
        return servletContext;
    }
 
    /**
     * Returns the framework's configurations as an Configuration object stored 
     * to the servlet context
     *
     * @return Configurations object
     */
    public Configurations getConfigurations() 
    {
        ServletContext servletContext = getServletContext();
        Configurations configurations = (Configurations)servletContext.getAttribute( Configurations.CONTEXT_NAME );
        return configurations;
        
    }


                  
    /**
     * Returns the name of the dedault admin group. This group has the rights
     * to add/remove/delete users to the system. Note that user doesn't need 
     * to be in default admin group to add new user groups, it is enought if
     * the user is in some of the admin groups (in certain node). In this case
     * the user can create and admin the nodes user groups.
     *
     * @return the nameo the user group
     */
    public String getDefaultAdminGroupName()
    {
        Configurations configurations = getConfigurations();
        
        //Lets get the default configurations (This contains the information
        //of the default admin group
        ConfigurationNode defaultNode = configurations.searchNode("/");
        String defaultAdminGroupName = defaultNode.getAdminGroupName();
        return defaultAdminGroupName;
    }

    public String getAdminGroupName( String path)
    {
        Configurations configurations = getConfigurations();
        
        //Lets get the default configurations (This contains the information
        //of the default admin group
        ConfigurationNode defaultNode = configurations.searchNode( path );
        if( defaultNode == null )
            return null;
        
        String defaultAdminGroupName = defaultNode.getAdminGroupName();
        return defaultAdminGroupName;
    }    
    /**
     * Returns a principal that represents the default admin group. This 
     * group can be used for example to check if the user belongs to certain 
     * admin group.
     */
    public FrameworkGroupPrincipal getDefaultAdminGroup()
    {
        try{
            
            String groupName = getDefaultAdminGroupName();    
            InitialContext ctx = new InitialContext();
            Object boundObj = ctx.lookup("UserFacadeEJB");
            UserFacadeHome home = (UserFacadeHome)PortableRemoteObject.narrow( boundObj, UserFacadeHome.class );
            UserFacade userFacade = home.create();
            
            FrameworkGroupPrincipal groupPrinciapl = userFacade.getGroupPrincipal( groupName, "/" );
            return groupPrinciapl;
            
        }catch( Exception e){
            return null;
        }
    }
    
    /**
     * Function search and returns the admin group for
     * certain node. If the group or node was not found the method
     * returns null value.
     *
     */
    public FrameworkGroupPrincipal getAdminGroup( String path ){
        
        try{
            String groupName = getAdminGroupName( path ); 
            if( groupName == null )
                return null;
            
            InitialContext ctx = new InitialContext();
            Object boundObj = ctx.lookup("UserFacadeEJB");
            UserFacadeHome home = (UserFacadeHome)PortableRemoteObject.narrow( boundObj, UserFacadeHome.class );
            UserFacade userFacade = home.create();
            
            FrameworkGroupPrincipal groupPrinciapl = userFacade.getGroupPrincipal( groupName, path );
            
            return groupPrinciapl;
            
        }catch( Exception e){
            return null;
        }        
    }
}
