/************************************************************************
 * Spooky - a service framework for the Internet
 * FrameworkGroupPrincipal.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.security.auth;


/**
 * Class represents a user group in the system. The user group can be used
 * to give priviledges to certain group of users. When the user is authenticated
 * the context of the user (<code>FrameworkContext</code>) gets a list of all
 * user groups available for the user. This list is updated when the user 
 * authenticate her self.
 *
 * @author  ekarlsso
 * @version 0.2
 */
public class FrameworkGroupPrincipal implements java.io.Serializable, java.security.Principal {

    private String groupName;
    private Integer groupId;
    
    /** Creates new FrameworkGroupPrincipal */
    public FrameworkGroupPrincipal() {
    }
    
    public FrameworkGroupPrincipal( String groupName ){
        this.groupName = groupName;
    }

    public FrameworkGroupPrincipal( Integer id, String groupName ){
        
        this.groupId = id;
        this.groupName = groupName;
    }    
    
    public boolean equals(java.lang.Object obj) 
    {
        if ((obj == null ) || !(obj instanceof FrameworkGroupPrincipal) )
            return false;
        
        FrameworkGroupPrincipal sp = (FrameworkGroupPrincipal)obj;
        if( groupId.intValue() == sp.getUserId() )
            return true;
        else
            return false;
    }
    
    
    public int getUserId()
    {
        return this.groupId.intValue();
    }
    
    public int getGroupId()
    {
        return this.groupId.intValue();
    }
    
    public java.lang.String getName() {
        return this.groupName;
    }
    
    public int hashCode() {
        return groupName.hashCode();
    }
    
    public java.lang.String toString() {
        return "Group name is: "+groupName;
    }
    
}
