/************************************************************************
 * Spooky - a service framework for the Internet
 * FrameowrkLoginModule.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.security.auth;

import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;
import java.util.*;

import spooky.users.ejb.*;

import javax.ejb.*;
import javax.naming.*;
import javax.rmi.*;

/**
 *
 * This class responsibility is to add all user group principals to the 
 * subject. This will also add the framework user principal to the subject.
 * if other is required then use other authentication system. (A Simple password 
 * module is provided also)
 *
 * @author  ekarlsso
 * @version 
 */
public class FrameworkLoginModule implements javax.security.auth.spi.LoginModule {

    private Subject subject;
    private CallbackHandler callbackHandler;
    private Map sharedState;
    private Map options;
    
    private String uniqueSessionId;
    private String username;
    private char[] passwd;
    
    //Authentication status
    private boolean status;
    private boolean comitStatus;
    
    private SystemPrincipal systemPrincipal;
    private FrameworkUserPrincipal userPrincipal;
    
    /** Creates new FrameworkLoginModule */
    public FrameworkLoginModule() {
    }

    /**
     * Function is called when authentication was unsuccesfully. Note that
     * any unsuccesfully authentication in any authentication module causes
     * this function to be called.
     *
     * @throws LoginException if there was a problem to complete this function 
     */
    public boolean abort() throws javax.security.auth.login.LoginException 
    {
        if( status == false )
        {
            return false;
        }
        else if( status == true && comitStatus == false )
        {
            status = false;
            username = null;
            
            if( passwd != null )
            {
                for( int i = 0; i < passwd.length; i++ )
                    passwd[i] = ' ';
                passwd = null;
            }
            
            systemPrincipal = null;
        }
        else
        {
            logout();
        }
        return true;
    }
    
    /**
     * Function is called if the authentication was completed succesfully.
     * This function adds all necerary principals to the user. These principals
     * includes the user groups and the user principal. Also a system principal
     * is added (it indicates a authentication session).
     *
     * @throws LoginExcpetion if there was a problem
     */
    public boolean commit() throws javax.security.auth.login.LoginException 
    {
        try 
        {
            if( status == false )
                return false;

            //First will we generate the system principal with
            //unique identifier. The system principal is not created
            //if it is allready created. Note that this indicates that
            //the there is another user database!
            Set systemPrincipalSet = subject.getPrincipals( SystemPrincipal.class );

            if( systemPrincipalSet.size() < 1 )
            {
                //Ok, there wasn't a authentication session id. Let's
                //create one!
                SystemPrincipal systemPrincipal = new SystemPrincipal("dasdada#da1132DDS");
                subject.getPrincipals().add( systemPrincipal );
            }

            InitialContext ctx = new InitialContext();
            Object boundObj = ctx.lookup("UserFacadeEJB");
            UserFacadeHome home = (UserFacadeHome)PortableRemoteObject.narrow( boundObj, UserFacadeHome.class );
            UserFacade userFacade = home.create();
            
            Set userPrincipalSet = subject.getPrincipals( FrameworkUserPrincipal.class );
            if( userPrincipalSet.size() < 1 )
            {
                FrameworkUserPrincipal userPrincipal = userFacade.getUserPrincipal( username );
                subject.getPrincipals().add( userPrincipal );
            }
            
            //Let's get the current user principal so that we can 
            //add add all user group principals
            userPrincipalSet = subject.getPrincipals( FrameworkUserPrincipal.class );
            Iterator princItr = userPrincipalSet.iterator();
            FrameworkUserPrincipal userPrincipal = (FrameworkUserPrincipal)princItr.next();
            
            //Then we will add all necesary user groups to the
            //suject. Note that this is done in every cases, event
            //the is another user database!
            Object mapObj = ctx.lookup("UserGroupMapEJB");
            UserGroupMapHome mapHome = (UserGroupMapHome)PortableRemoteObject.narrow( mapObj, UserGroupMapHome.class );
            String userName = userPrincipal.getName();
            Collection groups = mapHome.findByUserId( userName );
            int size = groups.size();
            
            Iterator mapItr = groups.iterator();

            while( mapItr.hasNext() )
            {
                UserGroupMap groupMap = (UserGroupMap)mapItr.next();
                FrameworkGroupPrincipal groupPrincipal = userFacade.getGroupPrincipal( groupMap.getGroupId() );
                String groupName = groupPrincipal.getName();
                subject.getPrincipals().add( groupPrincipal );
            }                
            

            //To Last we will clean up the parameters, so that they can 
            //not be readed from the memory.
            username = null;
            for( int i = 0; i < passwd.length; i++)
                passwd[i] = ' ';

            passwd = null;

            comitStatus = true;
            return true;
            
        }catch( Exception e ){
            String message = e.toString();
            throw new LoginException( message );
        }
    }
    
    /**
     *  Initializes the logic module.
     *
     *
     */
    public void initialize( Subject subject, CallbackHandler callbackHandler, Map map, Map map3) 
    {
        System.err.println("FrameworkLoginModule::initialize()");
        this.status = false;
        this.comitStatus = false;
        this.subject = subject;
        this.callbackHandler = callbackHandler;
        this.sharedState = map;
        this.options = map3;
    }
    
    /**
     * This method logins the user. The information used is stored within
     * the frameworks user principal information. Note that if other 
     * module is used to authenticate users (for example other users that
     * system administrators) a another login module must be provided that
     * adds also the system principal that describes a unique id for the 
     * atuehtnication session
     */
    public boolean login() throws javax.security.auth.login.LoginException 
    {
        //First we will get the information if there is also another
        //login module that can provide the authentication of 
        //users
        boolean anotherMethod = false;
        int sizeOfOptions = options.size();
        
        String loginMethod = (String)options.get("primary");
        if( loginMethod.toLowerCase().equals("true") )
            anotherMethod = true;
        
        //Let's check if there is a callback handler available
        if( callbackHandler == null )
            throw new LoginException("Error: no CallbackHandler available");

        //First will we create a callback's to retive the
        //required information (the username and the password name)
        Callback[] callbacks = new Callback[2];
        callbacks[0] = new NameCallback("Username: ");
        callbacks[1] = new PasswordCallback("Password: ", false );
        
        try 
        {
            callbackHandler.handle(callbacks);
            username = ((NameCallback)callbacks[0]).getName();
            char[] tmpPasswd = ((PasswordCallback)callbacks[1]).getPassword();
            
            if( tmpPasswd ==null )
                tmpPasswd = new char[0];

            passwd = new char[ tmpPasswd.length ];
            System.arraycopy( tmpPasswd, 0, passwd, 0, tmpPasswd.length );
            
            ((PasswordCallback)callbacks[1]).clearPassword();
            
        }catch( java.io.IOException ioe){
            throw new LoginException( ioe.toString() );
        }catch( UnsupportedCallbackException uce ){
            throw new LoginException("Error: "+uce.getCallback().toString()+" not available");
        }

        //Now we will put the password to string
        String passwordString = new String( passwd );
        try 
        {        
            //Now we will check if the there is a user
            //in the system and try to login the user.
            InitialContext ctx = new InitialContext();
            Object boundObj = ctx.lookup("UserFacadeEJB");
            UserFacadeHome home = (UserFacadeHome)PortableRemoteObject.narrow( boundObj, UserFacadeHome.class );
            UserFacade userFacade = home.create();
        
            if( userFacade.existUser( username ) )
            {
                //Ok! the user exist. Let's try to authenticate the
                //user with the password provided
                boolean authenticated = userFacade.matchUserWithPassword( username, passwordString );

                if( authenticated )
                {
                    //The password was right. Everything is ok!
                    status = true;
                    return true;
                }
                else
                {
                    //The password didn't match. Authentication failed
                    passwordString = null;
                    username = null;
                    for (int i = 0; i < passwd.length; i++)
                        passwd[i] = ' ';            
                    passwd = null;
                    status = false;
                    throw new FailedLoginException("Password was wrong!");                
                }
            }
            else
            {
                //The user didn't exist in the spooky frameworks
                //database
                if( anotherMethod )
                {
                    //There is another database too
                    status = true;
                    return true;
                }

                //No other database. Wich is very bad thing...
                passwordString = null;
                username = null;
                for (int i = 0; i < passwd.length; i++)
                    passwd[i] = ' ';            
                passwd = null;
                status = false;
                throw new FailedLoginException("User "+username+" was not found");
            }
            
        }catch( Exception e ){
            
            //An exception occured.
            passwordString = null;
            username = null;
            for (int i = 0; i < passwd.length; i++)
		passwd[i] = ' ';            
            passwd = null;
            status = false;            
            throw new FailedLoginException( e.toString() );
        }

    }
    
    /**
     *  Logout the user. Method removes the SystemPrincipal that
     *  was added by the commit method. Note that the system 
     *  principal contains an session id.
     *
     */
    public boolean logout() throws javax.security.auth.login.LoginException 
    {
        
        
        //First we will get a set of all principals available!
        Set systemPrincipals = subject.getPrincipals( SystemPrincipal.class );
        Set userPrincipals = subject.getPrincipals( FrameworkUserPrincipal.class );
        Set userGroupPrincipals = subject.getPrincipals( FrameworkGroupPrincipal.class );
        
        Iterator sysItr = systemPrincipals.iterator();
        while( sysItr.hasNext() )
        {
            SystemPrincipal prin = (SystemPrincipal)sysItr.next();
            subject.getPrincipals().remove( prin );
        }
        
        sysItr = userPrincipals.iterator();
        while( sysItr.hasNext() )
        {
            FrameworkUserPrincipal prin = (FrameworkUserPrincipal)sysItr.next();
            subject.getPrincipals().remove( prin );
        }

        sysItr = userPrincipals.iterator();
        while( sysItr.hasNext() )
        {
            FrameworkGroupPrincipal prin = (FrameworkGroupPrincipal)sysItr.next();
            subject.getPrincipals().remove( prin );
        }
        
        status = false;
        comitStatus = false;
        username = null;
        
        if( passwd != null )
        {
            for( int i = 0; i < passwd.length; i++ )
                passwd[i] = ' ';
            passwd = null;
        }
        
        systemPrincipal = null;
        return true;
    }
}
