/************************************************************************
 * Spooky - a service framework for the Internet
 * FrameworLoginContext.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.security.auth.login;

import javax.security.auth.*;
import javax.security.auth.callback.*;
import javax.security.auth.login.*;
import javax.security.auth.spi.*;
import com.sun.security.auth.login.*;


/**
 * Framework's login context module. This module supports the JAAS login context
 * module's API specification. 
 *
 * @author  ekarlsso
 * @version 
 */
public class FrameworkLoginContext  {

    private AppConfigurationEntry[] confEntry;
    private CallbackHandler callbackHandler;
    private String name;
    private Subject subject;
    private boolean[] status;
    private LoginModule[] loginModules;
    
    /**
     * Creates a new FrameworkLoginContext module. The name describes
     * the "configuration" entity to be used in current authentication
     * context. The callback handler describes the handler to be used
     *
     * @param name the name of the configruation
     * @param callbackHandler the used callback handler. The Spooky framework
     *                        uses the FrameworkCallbackHandler.
     */
    public FrameworkLoginContext( String name, CallbackHandler callbackHandler) 
    {
        //First we will load the basic objects
        this.callbackHandler = callbackHandler;
        this.subject = new Subject();
        ConfigFile cf = new ConfigFile();
        
        //Now we will get the configuration entity from overall configurations
        confEntry = cf.getAppConfigurationEntry(name);
        
        this.loginModules = new LoginModule[ confEntry.length ];
        this.status = new boolean[ confEntry.length ];
        
        for( int i = 0; i < status.length; i++ )
        {
            status[i] = false;
        }
    }
    
    /**
     * Method authenticates the user using all configurated login modules.
     * If the authentication was unsuccesfully a LoginException is thrown.
     */
    public void login() throws LoginException
    {
        try
        {
            //First we will complete the 2 phase
            //in authentication. This is done 
            //for each class.
            int i;
            for( i = 0; i < confEntry.length; i++ )
            {
                AppConfigurationEntry ap = confEntry[i];
                String className = ap.getLoginModuleName();
                loginModules[i] = (LoginModule)getClass().getClassLoader().loadClass( className ).newInstance();
                loginModules[i].initialize( subject, callbackHandler, null, ap.getOptions() );
                status[i] = loginModules[i].login();    
             }

            //Then the second phase in our authentication
            //process
            for( i = 0; i < confEntry.length; i++ )
            {
                if( status[i] ){
                    boolean bolValue = loginModules[i].commit();
                }
            }             
            
        }
        catch( LoginException le )
        {
            //Authentcation was unsuccesfully. Lets
            //abort all methods.
            for( int i = 0; i < confEntry.length; i++ )
            {
                if( status[i] )
                {
                     boolean bolValue =loginModules[i].abort();
                }
                status[i] = false;
            }              
                
            throw le;
        }catch( Exception e ){
            String message = e.toString();
            throw new LoginException( message );
        }
        
    }
    
    /**
     * Logouts the user.
     *
     */
    public void logout() throws LoginException
    {
        for( int i = 0; i < confEntry.length; i++ )
        {
            if( status[i] )
            {
                loginModules[i].logout();
            }
            status[i] = false;
        } 
    }

    /**
     * Returns the subject
     */
    public Subject getSubject() {
        return subject;
    }
}
