/************************************************************************
 * Spooky - a service framework for the Internet
 * UserFacadeEJB.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.users.ejb;

import spooky.sysservices.ejb.*;
import spooky.sysservices.*;
import spooky.security.auth.*;

import javax.ejb.*;
import javax.rmi.*;
import javax.naming.*;

import java.util.*;


/**
 * <p>
 * Class is a Facade to the user- and user group entity beans. The service
 * programmer should use this facade when using the entity beans so that
 * one can be sure that the service will function in future. Also this
 * facade bean contains logic to deal with transactions.
 * </p>
 * <p>
 * The UserFacadeEJB is an stateless entity bean
 * </p>
 *
 * @author ekarlsso
 * Created Sep 8, 2001 1:11:28 AM
 * Code generated by the Forte for Java EJB Module
 */

public class UserFacadeEJB implements SessionBean {
    
    private SessionContext context;

    /**
     * No argument constructor required by container.
     */
    public UserFacadeEJB() {
    }

    /**
     * Create method specified in EJB 1.1 section 6.10.3
     */
    public void ejbCreate() throws CreateException {
    }

    /* Methods required by SessionBean Interface. EJB 1.1 section 6.5.1. */

    /**
     * @see javax.ejb.SessionBean#setContext(javax.ejb.SessionContext)
     */
    public void setSessionContext(SessionContext context){
        this.context = context;
    }

    /**
     * @see javax.ejb.SessionBean#ejbActivate()
     */
    public void ejbActivate() {
    }

    /**
     * @see javax.ejb.SessionBean#ejbPassivate()
     */
    public void ejbPassivate() {
    }

    /**
     * @see javax.ejb.SessionBean#ejbRemove()
     */
    public void ejbRemove() {
    }
    
    /**
     * Creates a new user. The user will get a user name, full name and password.
     *
     * @param userName the user name of the user
     * @param fullName full name
     * @param password the user's password
     */
    public int createUser(java.lang.String userName, java.lang.String fullName, java.lang.String password) throws UserEjbException 
    {
        try 
        {
            InitialContext cx = new InitialContext();
            Object homeObj = cx.lookup("UserPrincipalEJB");
            UserPrincipalHome theHome = (UserPrincipalHome)PortableRemoteObject.narrow( homeObj, UserPrincipalHome.class );
            UserPrincipal userPrincipal = theHome.create( userName, fullName, password);
            return userPrincipal.getUserId();
            
        }catch( CreateException ce ){
            
            context.setRollbackOnly();
            throw new UserEjbException("Error creating the "+userName+": "+ce.toString() );
            
        }catch( Exception e ){
            
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );
        }
    }

    /**
     * Method removes an certain user.
     *
     */
    public void removeUser(java.lang.String userName) throws UserEjbException {
    
        try {
        
            InitialContext cx = new InitialContext();
            Object homeObj = cx.lookup("UserPrincipalEJB");
            UserPrincipalHome theHome = (UserPrincipalHome)PortableRemoteObject.narrow( homeObj, UserPrincipalHome.class );
            Collection usercoll = theHome.findByUserName( userName );
            
            if( usercoll.size() == 1 )
            {
                Iterator itr = usercoll.iterator();
                UserPrincipal userPrincipal = (UserPrincipal)itr.next();
                userPrincipal.remove();
            }
            else if( usercoll.size() > 1 )
                throw new UserEjbException("Error in database: too many user with username : "+userName);
            
        }catch( Exception e ) {
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );
        }
    }    
    
    /**
     * Method checks if there is an user named "userName"
     *
     * @param userName the user name
     */
    public boolean existUser(String userName) throws UserEjbException 
    {
        try
        {
            InitialContext cx = new InitialContext();
            Object homeObj = cx.lookup("UserPrincipalEJB");
            UserPrincipalHome theHome = (UserPrincipalHome)PortableRemoteObject.narrow( homeObj, UserPrincipalHome.class );
            Collection userCollection = theHome.findByUserName( userName );
            
            int collectionSize = userCollection.size();
            Iterator userItr = userCollection.iterator();
            
            while( userItr.hasNext() )
            {
                UserPrincipal userPrincipal = (UserPrincipal)userItr.next();
                if( userPrincipal.getUserName().equals( userName ) )
                    return true;
                else
                    throw new UserEjbException("The findByUserName method is not configurated propertly");
            }
            
            return false;
            
        }catch( FinderException fe ){
            return false;
        }catch( Exception e ){
            throw new UserEjbException( e.toString() );
        }
    }
    
    /***
     * Method creates a new user group for the framework. This user group is 
     * unit specific, but note that all admin groups can admin other groups 
     * below the unit.
     *
     * @param groupName the name of the group
     * @param path the unit path in the config tree
     * @param usersInGroup lists the users in the group (username, username2, username3 etc.)
     */
    public int createUserGroup(String groupName, java.lang.String path, java.lang.String explanation) throws UserEjbException {
    
        try 
        {
            //First we will get the new number for the
            //user group table
            InitialContext cx = new InitialContext();
            Object homeObj = cx.lookup("UserGroupEJB");
            UserGroupHome theHome = (UserGroupHome)PortableRemoteObject.narrow( homeObj, UserGroupHome.class );
            UserGroup userGroup = theHome.create( groupName, path, explanation ) ;        
            
            return userGroup.getGroupId();
            
        }catch( Exception e ){
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );
        }
        
    }
    
    /**
     * Removes certain user group from the data base. After the group has 
     * been removed a new group with the same name will not be consider as
     * the older group.
     */
    public void removeUserGroup(int groupId, String path) throws UserEjbException {
        
        try 
        {
            //First we will get the new number for the
            //user group table
            InitialContext cx = new InitialContext();
            Object homeObj = cx.lookup("UserGroupEJB");
            UserGroupHome theHome = (UserGroupHome)PortableRemoteObject.narrow( homeObj, UserGroupHome.class );
            UserGroup userGroup = theHome.findByPrimaryKey( new Integer( groupId ) );        
            userGroup.remove();
            
        }catch( Exception e ){
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );
        }        
        
    }
 

    
    
    /**
     * Returns certain user framework principal. The principal is indentified 
     * by the user name. Note that the system assumes that there is only one
     * user with the user name in the system (this must be true). Note that 
     * the id number of the user will not be same if there have been a user
     * with same user name before.
     *
     * if the user could not be returned method will return null. It will also
     * roll back the current transaction.
     *
     * @param userName the name of the user
     * @return user principal, if the user was not found a null is returned
     */
    public spooky.security.auth.FrameworkUserPrincipal getUserPrincipal(java.lang.String userName) {
        
        try {
            
            InitialContext cx = new InitialContext();
            Object boundObj = cx.lookup("UserPrincipalEJB");
            UserPrincipalHome home = (UserPrincipalHome)PortableRemoteObject.narrow( boundObj, UserPrincipalHome.class );
            
            Collection theCollection = home.findByUserName( userName );
            int size = theCollection.size();
            
            if( size < 1 ) {
                
                context.setRollbackOnly();
                return null;
            }
            
            Iterator itr = theCollection.iterator();
            UserPrincipal userPrincipal = (UserPrincipal)itr.next();
            
            String puserName = userPrincipal.getUserName();
            String fullName = userPrincipal.getFullName();
            Date created = userPrincipal.getCreated();
            Integer id = new Integer( userPrincipal.getUserId() );
            
            FrameworkUserPrincipal thePrincipal = new FrameworkUserPrincipal( id, userName, fullName, created );
            return thePrincipal;
            
        }catch( Exception e ){
            context.setRollbackOnly();
            return null;
        }
    }
    
    /**
     * Returns certain user group that can be found under certain path. Note 
     * that there can be many user groups that have same name butt different
     * path (tree path).
     *
     */
    public spooky.security.auth.FrameworkGroupPrincipal getGroupPrincipal(java.lang.String groupName) {
        
        try 
        {
            InitialContext cx = new InitialContext();
            Object boundObj = cx.lookup("UserGroupEJB");
            UserGroupHome home = (UserGroupHome)PortableRemoteObject.narrow( boundObj, UserGroupHome.class );
            
            Collection theCollection = home.findByGroupName( groupName );
            int size = theCollection.size();
            
            if( size < 1 ) {
                return null;
            }
            
            Iterator itr = theCollection.iterator();
            while( itr.hasNext() )
            {
                UserGroup userGroup = (UserGroup)itr.next();
                Integer id = new Integer( userGroup.getGroupId() );
                FrameworkGroupPrincipal thePrincipal = new FrameworkGroupPrincipal( id, groupName );
                return thePrincipal;
            }
            
            return null;
            
        }catch( Exception e ){
            context.setRollbackOnly();
            return null;
        }        
    }
    
    /**
     * Compares the user's password with the given password. If the password
     * matches value true is returned else false.
     *
     */
    public boolean matchUserWithPassword(java.lang.String userName, java.lang.String password) throws spooky.users.ejb.UserEjbException {
        
        try
        {
            InitialContext cx = new InitialContext();
            Object homeObj = cx.lookup("UserPrincipalEJB");
            UserPrincipalHome theHome = (UserPrincipalHome)PortableRemoteObject.narrow( homeObj, UserPrincipalHome.class );
            Collection userCollection = theHome.findByUserName( userName );
            
            if( userCollection.size() < 1 )
                return false;

            //Note that we will use the first user with the user name.
            Iterator itr = userCollection.iterator();
            UserPrincipal userPrincipal = (UserPrincipal)itr.next();
            
            if( userPrincipal.getPassword().equals( password ) )
                return true;
            
            return false;
            
        }catch( FinderException fe ){
            return false;
        }catch( Exception e ){
            throw new UserEjbException( e.toString() );
        }
    }
    
    public spooky.security.auth.FrameworkGroupPrincipal getGroupPrincipal(java.lang.String groupName, java.lang.String path) {
        try 
        {
            InitialContext cx = new InitialContext();
            Object boundObj = cx.lookup("UserGroupEJB");
            UserGroupHome home = (UserGroupHome)PortableRemoteObject.narrow( boundObj, UserGroupHome.class );
            
            Collection theCollection = home.findByGroupName( groupName );
            int size = theCollection.size();
            
            if( size < 1 ) {
                return null;
            }
            
            Iterator itr = theCollection.iterator();
            while( itr.hasNext() )
            {
                UserGroup userGroup = (UserGroup)itr.next();
                if( userGroup.getGroupPath().equals( path ) )
                {
                    Integer id = new Integer( userGroup.getGroupId() );
                    FrameworkGroupPrincipal thePrincipal = new FrameworkGroupPrincipal( id, groupName );
                    return thePrincipal;
                }
            }
            
            return null;
            
        }catch( Exception e ){
            context.setRollbackOnly();
            return null;
        }   
    }
    
    /**
     * Add's a new user to the group. If the user exist allready in the group 
     * nothing is done. If a exception is done, the transaction is rolled back.
     *
     */
    public boolean addUserToGroup(spooky.security.auth.FrameworkUserPrincipal userPrincipal, spooky.security.auth.FrameworkGroupPrincipal group) throws spooky.users.ejb.UserEjbException {
        
        try {
        
            InitialContext cx = new InitialContext();
            Object boundObj = cx.lookup("UserGroupMapEJB");
            UserGroupMapHome home = (UserGroupMapHome)PortableRemoteObject.narrow( boundObj, UserGroupMapHome.class );
            
            Collection groups = home.findByGroupId( group.getName() );
            Iterator itr = groups.iterator();
            
            //Let's check if there is allready a row for this group
            //that contains the user name.
            while( itr.hasNext() )
            {
                UserGroupMap groupMap = (UserGroupMap)itr.next();
                String userId = groupMap.getUserId();
                 
                if( userId.equals( userPrincipal.getName() ) )
                    return false;
            }
            
            UserGroupMap userGroupMap = home.create( group.getName(), userPrincipal.getName() );
            return true;
            
        }catch( Exception e ){
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );
        }
    }
    

    
    /**
     * Returns group principal
     */
    public spooky.security.auth.FrameworkGroupPrincipal getGroupPrincipal(java.lang.Integer groupId) {
    
        try 
        {
            InitialContext cx = new InitialContext();
            Object boundObj = cx.lookup("UserGroupEJB");
            UserGroupHome home = (UserGroupHome)PortableRemoteObject.narrow( boundObj, UserGroupHome.class );
            
            UserGroup userGroup = home.findByPrimaryKey(groupId);
            FrameworkGroupPrincipal thePrincipal = new FrameworkGroupPrincipal( groupId, userGroup.getGroupName()  );

            return thePrincipal;
            
        }catch( FinderException fe ){
            return null;
        }catch( Exception e ){
            context.setRollbackOnly();
            return null;
        }         
    }
    
    public boolean addUserToGroup(java.lang.String userName, java.lang.String groupName) throws spooky.users.ejb.UserEjbException {
        
        try 
        {
            InitialContext cx = new InitialContext();
            Object boundObj = cx.lookup("UserGroupMapEJB");
            UserGroupMapHome home = (UserGroupMapHome)PortableRemoteObject.narrow( boundObj, UserGroupMapHome.class );
            
            Collection groups = home.findByGroupId(groupName);
            Iterator itr = groups.iterator();
            
            //Let's check if there is allready a row for this group
            //that contains the user name.
            while( itr.hasNext() )
            {
                UserGroupMap groupMap = (UserGroupMap)itr.next();
                String userId = groupMap.getUserId();
                 
                if( userId.equals( userName ) )
                    return false;
            }
            
            UserGroupMap userGroupMap = home.create( groupName,  userName );
            return true;
            
        }catch( Exception e ){
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );
        }
    }
    
    /**
     * Removes an user from certain user group. 
     *
     */
    public boolean removeUserFromGroup(spooky.security.auth.FrameworkUserPrincipal userPrincipal, spooky.security.auth.FrameworkGroupPrincipal group) throws spooky.users.ejb.UserEjbException {
        
        try {

            InitialContext cx = new InitialContext();
            Object boundObj = cx.lookup("UserGroupMapEJB");
            UserGroupMapHome home = (UserGroupMapHome)PortableRemoteObject.narrow( boundObj, UserGroupMapHome.class );
            
            Collection groups = home.findByGroupId( group.getName() );
            Iterator itr = groups.iterator();
            
            while( itr.hasNext() )
            {
                UserGroupMap groupMap = (UserGroupMap)itr.next();
                if( groupMap.getUserId().equals( userPrincipal.getName() ) )
                {
                    groupMap.remove();
                    return true;
                }
            }
            
            return false;            
            
        }catch( Exception e ){
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );            
        }
    }    
    
    public boolean removeUserFromGroup(java.lang.String userName, java.lang.String group) throws spooky.users.ejb.UserEjbException {
        
    try {

            InitialContext cx = new InitialContext();
            Object boundObj = cx.lookup("UserGroupMapEJB");
            UserGroupMapHome home = (UserGroupMapHome)PortableRemoteObject.narrow( boundObj, UserGroupMapHome.class );
            
            Collection groups = home.findByGroupId( group );
            Iterator itr = groups.iterator();
            
            while( itr.hasNext() )
            {
                UserGroupMap groupMap = (UserGroupMap)itr.next();
                if( groupMap.getUserId().equals( userName ) )
                {
                    groupMap.remove();
                    return true;
                }
            }
            
            return false;            
            
        }catch( Exception e ){
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );            
        }
    }
    
    public boolean existGroup(java.lang.String groupName) {
        try
        {
            InitialContext cx = new InitialContext();
            Object homeObj = cx.lookup("UserGroupEJB");
            UserGroupHome theHome = (UserGroupHome)PortableRemoteObject.narrow( homeObj, UserGroupHome.class );
            Collection userCollection = theHome.findByGroupName( groupName );
            
            int collectionSize = userCollection.size();
            Iterator userItr = userCollection.iterator();
            
            while( userItr.hasNext() )
            {
                UserGroup groupPrincipal = (UserGroup)userItr.next();
                if( groupPrincipal.getGroupName().equals( groupName ) )
                    return true;
            }
            
            return false;
            
        }catch( FinderException fe ){
            return false;
        }catch( Exception e ){
            return false;
        }        
    }
    
    /**
     * Removes an user group. If the group was removed
     * a true is returned. If the group is not found
     * false is returned.
     */
    public boolean removeUserGroup(java.lang.String groupName) throws spooky.users.ejb.UserEjbException {
        
        try 
        {
            //First we will get the new number for the
            //user group table
            InitialContext cx = new InitialContext();
            Object homeObj = cx.lookup("UserGroupEJB");
            UserGroupHome theHome = (UserGroupHome)PortableRemoteObject.narrow( homeObj, UserGroupHome.class );
            Collection groups = theHome.findByGroupName( groupName );
            
            Iterator itr = groups.iterator();
            if( itr.hasNext() )
            {
                UserGroup userGroup = (UserGroup)itr.next();
                userGroup.remove();
                return true;
            }
            
            return false;
            
        }catch( Exception e ){
            context.setRollbackOnly();
            throw new UserEjbException( e.toString() );
        }            
    }
    
}