/************************************************************************
 * Spooky - a service framework for the Internet
 * ViewManager.java
 *
 * Copyright (C) 2001 Erik Karlsson.
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; version 2.1 of the 
 * License.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *************************************************************************/
package spooky.view.web;

import spooky.config.*;
import java.util.*;

/**
 *
 * @author  Erik Karlsson
 * @version 
 */
public class ViewManager implements java.io.Serializable {

    public static String CONTEXT_NAME = "view-manager";
    
    private HashMap theConfigurations;
    private HashMap pageElements;
    private String currentView;
    
    
    /** Creates new ViewManager */
    public ViewManager( HashMap theConfigurations ) {
        this.pageElements = new HashMap();
        this.theConfigurations = theConfigurations;
    }

    /**
     * returns the template name of the requested view. The template name
     * is searched from the configuration file. If the template is not
     * forn an exception will be trhown.
     *
     * @param view the name of the view
     * @return the template name
     */
    public String getTemplate( String view ) throws ViewManagerException{
        
        try {
            System.err.println("getTemplate::view-attribute:: "+view);
            ViewConfig viewConfig  = (ViewConfig)theConfigurations.get( view );
            if( viewConfig != null ) {

                String nextView = viewConfig.getTemplate();
                HashMap currentPageElements = viewConfig.getPageElements();

                pageElements.put( viewConfig.getName(), currentPageElements );
                currentView = viewConfig.getName();

                return nextView;

            }else {
                throw new ViewManagerException("ViewManager::getTemplate::Configuration for the selected view was not found");
            }
            
        }catch( Exception e ){
            throw new ViewManagerException("getTemplate:: "+e.toString() );
        }
    }
    
    public PageElementConfig getPageElementConfig( String name )
    {
        HashMap currentPageElements = (HashMap)pageElements.get( currentView );
        return (PageElementConfig)currentPageElements.get( name );        
    }
    /**
     * Returns the next .jsp page corresponding to the name.
     */
    public PageElementConfig getNextJSP( String name ) {

        HashMap currentPageElements = (HashMap)pageElements.get( currentView );
        return (PageElementConfig)currentPageElements.get( name );
    }
}
